#ifndef _SILECS_PLCHANDLER_H_
#define _SILECS_PLCHANDLER_H_

#include <map>
#include <string>

namespace Silecs
{

class PLC;

/*!
 * \class PLCHandler
 * \brief Takes care of managing all the PLC objects. Provides a way to access the PLCs, in addition
 * it also provides some send and receive functionality that run in parallel on all PLCs.
 */
class PLCHandler
{
public:
    PLCHandler();

    PLC& addPLC(const std::string& plcId, const std::string& parameterFile="");

    PLC& getPLC(const std::string& plcId, const std::string& parameterFile="");

    bool isExistingPLC(const std::string& plcId) const;

    const std::string& getHostname() const;

    /*!
     * \brief Acquires one particular registers block of all devices of all enabled PLCs of that cluster.
     * The method tries to (re)connect each PLC if the connection is not established yet or unexpectedly interrupted (network failure, PLC down, etc.).
     * \param blockName name of the block to be acquired
     */
    void recv(const std::string& blockName);

    /*!
     * \brief Transmits the registers block to all devices of all enabled PLCs of that cluster.
     * The method tries to (re)connect each PLC if the connection is not established yet or unexpectedly interrupted (network failure, PLC down, etc.).
     * \param blockName name of the block to be transmitted
     */
    void send(const std::string& blockName);

    /*!
     * \brief Copies input buffer in output buffer
     * The method copies the input buffer in the output buffer for each READ-WRITE register register in the cluster for the block passed as parameter
     * \param blockName name of the block for which the copy must be performed
     */
    void copyInToOut(const std::string& blockName);

private:

    void getHostNameByAddress(const std::string& ipAddress, std::string& hostName) const;
    void getAddressByHostName(const std::string& hostName, std::string& ipAddress) const;

    std::string fecHostName_;
    std::map<std::string, PLC> plcMap_;
    std::map<std::string, std::string> plcIpMap_;
};

} // namespace

#endif // _SILECS_PLCHANDLER_H_
